import { Telemetry, collect } from '@vuedx/shared';
import { createTypescriptLanguageService } from '@vuedx/vue-languageservice';
import * as Path from 'path';
import { performance } from 'perf_hooks';

const PATCHED_METHODS = Symbol('Vue Patched Methods');
function overrideMethod(target, methodName, createOverride) {
    var _a;
    const patched = (_a = target[PATCHED_METHODS]) !== null && _a !== void 0 ? _a : [];
    if (patched.includes(methodName)) {
        return () => {
            var _a;
            // @ts-expect-error
            if (((_a = target[methodName]) === null || _a === void 0 ? void 0 : _a._original) != null) {
                // @ts-expect-error
                target[methodName] = target[methodName]._original;
            }
        };
    }
    let fn = target[methodName];
    if (typeof fn === 'function') {
        try {
            fn = fn.bind(target);
        }
        catch {
            // - ignore
        }
    }
    target[methodName] = createOverride(fn);
    // @ts-expect-error
    target[methodName]._original = fn;
    patched.push(methodName);
    target[PATCHED_METHODS] = patched;
    return () => {
        target[methodName] = fn;
    };
}

var version = "0.7.4";

/* eslint-disable import/first */
console.log = console.info = console.debug = console.error;
const services = new WeakMap();
function init({ typescript }) {
    Telemetry.setup('https://a1461052e1d94c7a9ee7c3f7add71b24@o237831.ingest.sentry.io/5595721', 'typescript-plugin-vue', version, 0.001, // 1 in thousand
    { typescriptVersion: typescript.versionMajorMinor });
    overrideMethod(typescript, 'getSupportedExtensions', (fn) => (options, extraFileExtensions) => {
        const extensions = fn(options, extraFileExtensions);
        const index = extensions.indexOf('.vue');
        if (index >= 0) {
            // File extensions are sorted in order of their priorities.
            // We need to put .vue before .ts
            extensions.splice(index, 1);
            extensions.unshift('.vue');
        }
        return extensions;
    });
    return {
        create(info) {
            const id = Date.now();
            const start = performance.now();
            const currentService = info.languageService;
            if (currentService._isVueTS)
                return info.languageService;
            info.project.projectService.logger.info(`[VueDX] (plugin) ${id} Plugin start`);
            patchExtraFileExtensions(info.project);
            const prevService = services.get(info.project);
            if (prevService != null)
                return prevService;
            const service = createTypescriptLanguageService({
                ...info,
                typescript,
                typesDir: Path.resolve(__dirname, '..', 'runtime'),
            });
            services.set(info.project, service);
            const end = performance.now();
            collect('plugin_create', {
                startupTime: end - start,
            });
            info.project.projectService.logger.info(`[VueDX] (plugin) ${id} Plugin started in ${end - start}ms`);
            return service;
        },
        getExternalFiles(project) {
            var _a, _b;
            return (_b = (_a = services.get(project)) === null || _a === void 0 ? void 0 : _a.getExternalFiles(project)) !== null && _b !== void 0 ? _b : [];
        },
        onConfigurationChanged(_config) { },
    };
}
function patchExtraFileExtensions(project) {
    const projectService = project.projectService;
    const extraFileExtensions = [
        {
            extension: '.vue',
            isMixedContent: false,
            scriptKind: 7, // deferred extra extensions are included in root files search.
        },
    ];
    overrideMethod(projectService, 'setHostConfiguration', (setHostConfiguration) => {
        return (args) => {
            projectService.logger.info('[VueDX] setHostConfiguration: ' + JSON.stringify(args));
            const current = projectService
                .hostConfiguration.extraFileExtensions;
            projectService.logger.info('Current Extra Extensions: ' + JSON.stringify(current));
            if (args.extraFileExtensions != null) {
                args.extraFileExtensions.push(...extraFileExtensions);
            }
            else if (current == null) ;
            else if (current.every((ext) => ext.extension !== '.vue')) {
                args.extraFileExtensions = [...current, ...extraFileExtensions];
            }
            projectService.logger.info('New Extra Extensions: ' + JSON.stringify(args.extraFileExtensions));
            return setHostConfiguration(args);
        };
    });
    const current = projectService
        .hostConfiguration.extraFileExtensions;
    if (Array.isArray(current) &&
        current.some((ext) => ext.extension === '.vue')) ;
    else if (Array.isArray(current)) {
        // Enable .vue after enhancing the language server.
        projectService.logger.info('Add extra extensions now.');
        projectService.setHostConfiguration({});
        return true;
    }
    return false;
}

export default init;
//# sourceMappingURL=index.mjs.map
